import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the total value to show on the Areachart
// This function formats the number to a human-readable format with abbreviated units (e.g. 1M, 2K, 3B).
const xAxisTickFormat = number =>
    format('.3s')(number)

const render = data => {
  
  const xValue = d => d.Year;
  const xAxisLabel = 'Years';
  
  const yValue = d => d.Units_sold;
  const circleRadius = 7;
  const yAxisLabel = 'Units_Sold (M)';
  
  const margin = { top: 10, right: 40, bottom: 88, left: 150 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;  
  // Determing the Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  // Determing the Scale for Y axis  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  // POsitioning the svg element   
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  // Assigning values to x axis 
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  // Assigning values to y axis  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(xAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 65)
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
  // To draw the line on the curve   and adding it to svg 
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  // To draw the area under the curve and adding it to svg 
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
g.append('path')
  	.attr('class', 'line-chart') // assigns the 'line-chart' class to this path
  	.attr('d', lineGenerator(data)); // assigns the path's shape using the line generator and the input data

g.append('path')
  	.attr('class', 'area-chart') // assigns the 'area-chart' class to this path
  	.attr('d', areaGenerator(data)); // assigns the path's shape using the area generator and the input data

};
// Loading and Parsing CSV data
csv('https://gist.githubusercontent.com/anvesh2129/83889c52fcf0e6e3ce63a0dcd495694d/raw/9a3a3b1ee9f29db0871e43e161a49ac4b916fa91/Playstation_sales.csv')
  .then(data => {
    data.forEach(d => {
      d.Units_sold = +d.Units_sold;
//Formatting to retrieve year from the date
      d.Year = d3.timeParse("%Y")(d.Year);
    });
    render(data);
  	console.log(data);
  });